@@ -17,7 +17,6 @@
 import time
 import unittest
 from testconfig import config
-
 from test import HPE3ParClient_base as hpe3parbase
 
 from hpe3parclient import exceptions
@@ -42,13 +41,23 @@ REMOTE_COPY_TARGETS = [{"targetName": "testTarget",
                         "roleReversed": False,
                         "groupLastSyncTime": None}]
 RC_VOLUME_NAME = 'RC_VOLUME1_UNIT_TEST' + hpe3parbase.TIME
+SCHEDULE_NAME1 = 'SCHEDULE_NAME1' + hpe3parbase.TIME
+SCHEDULE_NAME2 = 'SCHEDULE_NAME2' + hpe3parbase.TIME
 SKIP_RCOPY_MESSAGE = ("Only works with flask server.")
 SKIP_FLASK_RCOPY_MESSAGE = ("Remote copy is not configured to be tested "
                             "on live arrays.")
+TARGET_NAME = 'testtarget'
+SOURCE_PORT = '1:1:1'
+TARGET_PORT = '10.10.10.1'
 RCOPY_STARTED = 3
 RCOPY_STOPPED = 5
 FAILOVER_GROUP = 7
 RESTORE_GROUP = 10
+MODE = 'sync'
+VOLUME_PAIR_LIST = {'volumePairs': [{'sourceVolumeName': 'primary_vol1',
+                                     'targetVolumeName': 'secondary_vol1'},
+                                    {'sourceVolumeName': 'primary_vol2',
+                                     'targetVolumeName': 'secondary_vol2'}]}
 
 
 def is_live_test():
@@ -2063,6 +2072,222 @@ class HPE3ParClientVolumeTestCase(hpe3parbase.HPE3ParClientBaseTestCase):
 
         self.printFooter('set_qos')
 
+    def test_25_promote_virtual_copy(self):
+        self.printHeader('promote_virtual_copy')
+
+        optional = {'snapCPG': CPG_NAME1}
+        self.cl.createVolume(VOLUME_NAME1, CPG_NAME1, SIZE, optional)
+        # add one
+        self.cl.createSnapshot(SNAP_NAME1, VOLUME_NAME1)
+        # no API to get and check
+
+        resp = self.cl.promoteVirtualCopy(SNAP_NAME1)
+        self.assertIsNotNone(resp['taskid'])
+
+        self.printFooter('promote_virtual_copy')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test_25_promote_virtual_copy_on_replicated_volume(self):
+        self.printHeader('promote_virtual_copy_on_replicated_volume')
+
+        # Create empty remote copy group
+        self.cl.createRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1,
+                                      REMOTE_COPY_TARGETS,
+                                      optional={"domain": DOMAIN})
+
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        self.assertEqual(REMOTE_COPY_GROUP_NAME1, resp['name'])
+
+        # Create volume
+        optional = {'comment': 'test volume', 'tpvv': True}
+        self.cl.createVolume(RC_VOLUME_NAME, CPG_NAME1, SIZE, optional)
+
+        # Add volume to remote copy group
+        self.cl.addVolumeToRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1,
+                                           RC_VOLUME_NAME,
+                                           REMOTE_COPY_TARGETS)
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        volumes = resp['volumes']
+        self.assertEqual(RC_VOLUME_NAME, volumes[0]['name'])
+
+        self.cl.createSnapshot(SNAP_NAME1, RC_VOLUME_NAME)
+
+        # Stop remote copy for the group
+        self.cl.stopRemoteCopy(REMOTE_COPY_GROUP_NAME1)
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        targets = resp['targets']
+        self.assertEqual(RCOPY_STOPPED, targets[0]['state'])
+
+        optional = {'allowRemoteCopyParent': True}
+        resp = self.cl.promoteVirtualCopy(SNAP_NAME1, optional)
+        self.assertIsNotNone(resp['taskid'])
+
+        # Start remote copy for the group
+        self.cl.startRemoteCopy(REMOTE_COPY_GROUP_NAME1)
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        targets = resp['targets']
+        self.assertEqual(RCOPY_STARTED, targets[0]['state'])
+
+        self.printFooter('promote_virtual_copy_on_replicated_volume')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test_25_promote_virtual_copy_with_bad_param(self):
+        self.printHeader('promote_virtual_copy_with_bad_param')
+
+        self.cl.createVolume(VOLUME_NAME1, CPG_NAME1, SIZE)
+        # add one
+        self.cl.createSnapshot(SNAP_NAME1, VOLUME_NAME1)
+        # no API to get and check
+
+        optional = {'online': True,
+                    'allowRemoteCopyParent': True,
+                    'priority': 1}
+        self.assertRaises(
+            exceptions.HTTPConflict,
+            self.cl.promoteVirtualCopy,
+            SNAP_NAME1,
+            optional
+        )
+
+        self.printFooter('promote_virtual_copy_with_bad_param')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test_25_promote_vcopy_on_rep_vol_with_bad_param(self):
+        self.printHeader('promote_vcopy_on_rep_vol_with_bad_param')
+
+        # Create empty remote copy group
+        self.cl.createRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1,
+                                      REMOTE_COPY_TARGETS,
+                                      optional={"domain": DOMAIN})
+
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        self.assertEqual(REMOTE_COPY_GROUP_NAME1, resp['name'])
+
+        # Create volume
+        optional = {'comment': 'test volume', 'tpvv': True}
+        self.cl.createVolume(RC_VOLUME_NAME, CPG_NAME1, SIZE, optional)
+
+        # Add volume to remote copy group
+        self.cl.addVolumeToRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1,
+                                           RC_VOLUME_NAME,
+                                           REMOTE_COPY_TARGETS)
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        volumes = resp['volumes']
+        self.assertEqual(RC_VOLUME_NAME, volumes[0]['name'])
+
+        self.cl.createSnapshot(SNAP_NAME1, RC_VOLUME_NAME)
+
+        # Stop remote copy for the group
+        self.cl.stopRemoteCopy(REMOTE_COPY_GROUP_NAME1)
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        targets = resp['targets']
+        self.assertEqual(RCOPY_STOPPED, targets[0]['state'])
+
+        self.assertRaises(
+            exceptions.HTTPForbidden,
+            self.cl.promoteVirtualCopy,
+            SNAP_NAME1,
+            optional
+        )
+
+        # Start remote copy for the group
+        self.cl.startRemoteCopy(REMOTE_COPY_GROUP_NAME1)
+        resp = self.cl.getRemoteCopyGroup(REMOTE_COPY_GROUP_NAME1)
+        targets = resp['targets']
+        self.assertEqual(RCOPY_STARTED, targets[0]['state'])
+
+        self.printFooter('promote_vcopy_on_rep_vol_with_bad_param')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test26_admit_rcopy_link(self):
+        self.printHeader('admit_rcopy_link_test')
+        res = self.cl.admitRemoteCopyLinks(TARGET_NAME,
+                                           SOURCE_PORT,
+                                           TARGET_PORT)
+        self.assertEqual(res, [])
+        self.printFooter('admit_rcopy_link_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test27_dismiss_rcopy_link(self):
+        self.printHeader('dismiss_rcopy_link_test')
+        res = self.cl.dismissRemoteCopyLinks(TARGET_NAME,
+                                             SOURCE_PORT,
+                                             TARGET_PORT)
+        self.assertEqual(res, [])
+        self.printFooter('dismiss_rcopy_link_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test28_start_rcopy(self):
+        self.printHeader('start_rcopy_test')
+        res = self.cl.startrCopy()
+        self.assertEqual(res, [])
+        self.printFooter('start_rcopy_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test29_admit_rcopy_target(self):
+        self.printHeader('admit_rcopy_target_test')
+        res = self.cl.admitRemoteCopyTarget(TARGET_NAME,
+                                            MODE,
+                                            REMOTE_COPY_GROUP_NAME1)
+        self.assertEqual(res, [])
+        self.printFooter('admit_rcopy_target_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test30_admit_rcopy_target(self):
+        self.printHeader('admit_rcopy_target_test')
+        res = self.cl.admitRemoteCopyTarget(TARGET_NAME,
+                                            MODE,
+                                            REMOTE_COPY_GROUP_NAME1,
+                                            VOLUME_PAIR_LIST)
+        self.assertEqual(res, [])
+        self.printFooter('admit_rcopy_target_test')
+
+    # TODO: Fix this later
+    # @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    # def test31_dismiss_rcopy_target(self):
+    #    self.printHeader('dismiss_rcopy_target_test')
+    #    res = self.cl.dismissRemoteCopyTarget(TARGET_NAME,
+    #                                          REMOTE_COPY_GROUP_NAME1)
+    #    self.assertEqual(res, [])
+    #    self.printFooter('dismiss_rcopy_target_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test32_create_schedule(self):
+        self.printHeader('create_schedule_test')
+        cmd = "createsv -ro snap-" + VOLUME_NAME1 + " " + VOLUME_NAME1
+        res = self.cl.createSchedule(SCHEDULE_NAME1, cmd, 'hourly')
+        self.assertEqual(res, None)
+        self.printFooter('create_schedule_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test33_delete_schedule(self):
+        self.printHeader('delete_schedule_test')
+        res = self.cl.deleteSchedule(SCHEDULE_NAME1)
+        self.assertEqual(res, None)
+        self.printFooter('delete_schedule_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test34_modify_schedule(self):
+        self.printHeader('modify_schedule_test')
+        res = self.cl.modifySchedule(SCHEDULE_NAME1,
+                                     {'newName': SCHEDULE_NAME2})
+        self.assertEqual(res, None)
+        self.printFooter('modify_schedule_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test35_suspend_schedule(self):
+        self.printHeader('suspend_schedule_test')
+        res = self.cl.suspendSchedule(SCHEDULE_NAME1)
+        self.assertEqual(res, None)
+        self.printFooter('suspend_schedule_test')
+
+    @unittest.skipIf(is_live_test(), SKIP_RCOPY_MESSAGE)
+    def test36_resume_schedule(self):
+        self.printHeader('resume_schedule_test')
+        res = self.cl.resumeSchedule(SCHEDULE_NAME1)
+        self.assertEqual(res, None)
+        self.printFooter('resume_schedule_test')
+
 # testing
 # suite = unittest.TestLoader().
 #   loadTestsFromTestCase(HPE3ParClientVolumeTestCase)
